library(tidyverse)
library(mice)
library(miceadds)

args <- commandArgs(trailingOnly = TRUE)

inputfile  <- args[[1]] # "../output/v1.1/data/validation/df_train_mi_1_fold_1.rds"#
outputfile <- args[[2]]

config <- yaml::read_yaml("config.yml")



tbl_train <- readRDS(inputfile) %>%
    as_tibble()

tbl_blank <- tbl_train %>%
    select(
        -Outcomes.DerivedCompositeGOSE,
        -Outcomes.DerivedCompositeGOSEDaysPostInjury
    ) %>%
    distinct() %>%
    mutate(
        Outcomes.DerivedCompositeGOSE = NA,
        Outcomes.DerivedCompositeGOSEDaysPostInjury = 180
    )

tbl_gose <- tbl_train %>%
    bind_rows(tbl_blank) %>%
    select(
        gupi,
        Outcomes.DerivedCompositeGOSEDaysPostInjury,
        Outcomes.DerivedCompositeGOSE
    )

tbl_baseline <- tbl_train %>%
    bind_rows(tbl_blank) %>%
    select(
        -Outcomes.DerivedCompositeGOSEDaysPostInjury,
        -Outcomes.DerivedCompositeGOSE
    ) %>%
    group_by(gupi) %>%
    summarize(
        across(everything(), first),
        .groups = "drop"
    )

map_to_timepoint <- function(x,
                             timepoints = c(14, 90, 180, 365)
                             ) {
    xx <- matrix(
        rep(x, length(timepoints)),
        ncol = length(timepoints)
    )
    as.character(timepoints)[
        abs(xx - matrix(
            rep(timepoints, length(x)),
            ncol = length(timepoints),
            byrow = TRUE
        )) %>%
        apply(1, which.min)
    ]
}

tbl_combined <- expand_grid(
        gupi = tbl_gose$gupi %>% unique,
        timepoint = c("14", "90", "180", "365")
    ) %>%
    left_join(
        tbl_gose %>%
            mutate(
                     gose = as.integer(as.character(Outcomes.DerivedCompositeGOSE)),
                timepoint = map_to_timepoint(Outcomes.DerivedCompositeGOSEDaysPostInjury)
            ) %>%
            group_by(gupi, timepoint) %>%
            arrange(
                gupi,
                as.integer(timepoint),
                abs(Outcomes.DerivedCompositeGOSEDaysPostInjury - as.integer(timepoint))
            ) %>%
            select(
                -Outcomes.DerivedCompositeGOSEDaysPostInjury,
                -Outcomes.DerivedCompositeGOSE
            ) %>%
            summarize(
                gose = first(gose[!is.na(gose)]),
                .groups = "drop"
            ),
        by = c("gupi", "timepoint")
    ) %>%
    mutate(
        gose = factor(gose, levels = c(1, 3:8), ordered = TRUE)
    ) %>%
    mutate(
        timepoint = sprintf("gose_%s", timepoint)
    ) %>%
    pivot_wider(
         names_from = timepoint,
        values_from = gose
    ) %>%
    left_join(
        tbl_baseline,
        by = "gupi"
    )

predmat <- mice::make.predictorMatrix(data = tbl_combined)
predmat[ , "gupi"] <- 0

method <- mice::make.method(data = tbl_combined)

tbl_imputed <- tbl_combined %>%
    as.data.frame() %>%
    mice::mice(
                 method = method,
        predictorMatrix = predmat,
                      m = 100,
                maxiter = 25,
                   seed = 42
    )


tbl_imputed <- bind_rows(
        complete(tbl_imputed, "long") %>%
            as_tibble() ,
        complete(tbl_imputed, 0) %>%
            as_tibble %>%
            mutate(.imp = 0, .id = row_number())
    ) %>%
    arrange(.imp, .id) %>%
    pivot_longer(
        starts_with("gose_"),
        names_to = "tmp",
        values_to = "GOSE"
    ) %>%
    separate(tmp, c("tmp", "timepoint")) %>%
    select(.imp, gupi, timepoint, GOSE)


tbl_posteriors <- expand_grid(
        gupi = unique(tbl_combined$gupi),
        GOSE = c(1:8)
    ) %>%
    left_join(
        tbl_imputed %>%
            filter(timepoint == 180, .imp > 0) %>%
            group_by(gupi, timepoint, GOSE) %>%
            dplyr::summarize(
                t = 180,
                n = dplyr::n(),
                .groups = "drop"
            ) %>%
            group_by(gupi) %>%
            mutate(
                probability = n / sum(n),
                GOSE = as.integer(as.character(GOSE))
            ) %>%
            ungroup() %>%
            arrange(gupi, GOSE) %>%
            select(gupi, GOSE, probability),
        by = c("gupi", "GOSE")
    ) %>%
    mutate(
        t = 180L,
        probability = if_else(is.na(probability), 0, probability)
    ) %>%
    select(gupi, GOSE, probability, t)


saveRDS(tbl_posteriors, outputfile)



# tmp2 %>%
#     filter(
#         gupi %in% tbl_test$gupi
#     ) %>%
#     group_by(gupi) %>%
#     filter(cur_group_id() <= 36) %>%
#     ungroup() %>%
#     {ggplot(filter(., .imp == 0, !is.na(gose))) +
#         aes(days, gose) +
#         geom_point(data = filter(., .imp > 0, timepoint == "180"), color = "red", alpha = 1/25) +
#         geom_point() +
#         facet_wrap(~gupi)
#     }
